int var_smtpd_tls_scache_timeout;
int var_smtpd_policy_idle;
int var_smtp_sasl_auth_cache_time;
int var_min_backoff_time;
int var_smtpd_uproxy_tmout;
int var_verify_sender_ttl;
int var_smtpd_policy_tmout;
int var_xport_refill_delay;
int var_psc_barlf_ttl;
int var_psc_dnsbl_max_ttl;
int var_psc_pregr_ttl;
int var_xport_rate_delay;
int var_dnsblog_delay;
int var_verify_pos_exp;
int var_tlsp_watchdog;
int var_tls_prng_exch_period;
int var_smtpd_tmout;
int var_psc_cmd_time;
int var_milt_conn_time;
int var_psc_uproxy_tmout;
int var_psc_watchdog;
int var_fflush_purge;
int var_milt_cmd_time;
int var_delay_warn_time;
int var_smtp_conn_tmout;
int var_psc_greet_wait;
int var_qmgr_daemon_timeout;
int var_anvil_time_unit;
int var_verify_neg_try;
int var_verify_poll_delay;
int var_psc_pipel_ttl;
int var_dest_rate_delay;
int var_max_backoff_time;
int var_verify_scan_cache;
int var_verify_pos_try;
int var_smtpd_proxy_tmout;
int var_smtpd_err_sleep;
int var_smtp_rset_tmout;
int var_smtp_xfwd_tmout;
int var_tls_reseed_period;
int var_smtpd_policy_try_delay;
int var_smtpd_starttls_tmout;
int var_scache_proto_tmout;
int var_smtp_reuse_time;
int var_smtp_cache_conn;
int var_smtp_mail_tmout;
int var_qmgr_ipc_timeout;
int var_smtp_starttls_tmout;
int var_smtp_quit_tmout;
int var_smtp_helo_tmout;
int var_postlogd_watchdog;
int var_throttle_time;
int var_lmtp_tls_scache_timeout;
int var_smtp_tls_scache_timeout;
int var_smtp_pix_thresh;
int var_smtp_data2_tmout;
int var_smtp_data1_tmout;
int var_smtp_data0_tmout;
int var_qmqpd_err_sleep;
int var_transport_retry_time;
int var_queue_run_delay;
int var_command_maxtime;
int var_milt_msg_time;
int var_anvil_stat_time;
int var_verify_neg_exp;
int var_smtpd_policy_ttl;
int var_smtp_pix_delay;
int var_smtp_rcpt_tmout;
int var_qmqpd_timeout;
int var_psc_dnsbl_tmout;
int var_fflush_refresh;
int var_scache_stat_time;
int var_scache_ttl_lim;
int var_psc_cache_scan;
int var_psc_cache_ret;
int var_psc_nsmtp_ttl;
int var_psc_dnsbl_min_ttl;
int var_qmgr_clog_warn_time;
int var_dsn_queue_time;
int var_max_queue_time;
