bool var_smtpd_peername_lookup;
bool var_smtpd_tls_ask_ccert;
bool var_smtpd_sasl_auth_hdr;
bool var_smtp_tls_conn_reuse;
bool var_smtp_defer_mxaddr;
bool var_mailtool_compat;
bool var_smtpd_tls_received_header;
bool var_smtpd_tls_enable_rpk;
bool var_smtp_tls_note_starttls_offer;
bool var_smtp_send_xforward;
bool var_smtp_quote_821_env;
bool var_resolve_nulldom;
bool var_smtpd_delay_open;
bool var_smtpd_tls_req_ccert;
bool var_ign_srv_lookup_err;
bool var_smtp_cache_demand;
bool var_psc_pipel_enable;
bool var_disable_vrfy_cmd;
bool var_strict_mbox_owner;
bool var_cleanup_mask_stray_cr_lf;
bool var_auto_8bit_enc_hdr;
bool var_verp_bounce_off;
bool var_percent_hack;
bool var_smtp_balance_inet_proto;
bool var_smtp_tls_enable_rpk;
bool var_smtp_tls_force_tlsa;
bool var_smtp_enforce_tls;
bool var_exp_own_alias;
bool var_smtpd_tls_set_sessid;
bool var_smtpd_rej_unl_from;
bool var_smtp_bind_addr_enforce;
bool var_helo_required;
bool var_reset_owner_attr;
bool var_resolve_dequoted;
bool var_append_at_myorigin;
bool var_smtpd_tls_wrappermode;
bool var_smtp_always_ehlo;
bool var_skip_quit_resp;
bool var_allow_min_user;
bool var_smtpd_client_port_log;
bool var_smtpd_tls_auth_only;
bool var_broken_auth_clients;
bool var_smtpd_sasl_enable;
bool var_smtpd_delay_reject;
bool var_smtp_never_ehlo;
bool var_psc_nsmtp_enable;
bool var_smtpd_enforce_tls;
bool var_force_mime_iconv;
bool var_always_add_hdrs;
bool var_smtp_tls_enforce_peername;
bool var_smtp_sasl_enable;
bool var_smtp_skip_5xx_greeting;
bool var_allow_untrust_route;
bool var_allow_srv_fallback;
bool var_smtp_tls_wrappermode;
bool var_strict_rfc821_env;
bool var_smtp_use_tls;
bool var_qmqpd_client_port_log;
bool var_psc_barlf_enable;
bool var_dsn_delay_cleared;
bool var_conc_feedback_debug;
bool var_stat_home_dir;
bool var_resolve_num_dom;
bool var_smtpd_rej_unl_rcpt;
bool var_show_unk_rcpt_table;
bool var_ign_mx_lookup_err;
bool var_lmtp_assume_final;
bool var_smtp_sasl_auth_soft_bounce;
bool var_smtp_sender_auth;
bool var_smtp_tls_blk_early_mail_reply;
bool var_smtpd_use_tls;
bool var_swap_bangpath;
bool var_smtpd_forbid_unauth_pipe;
bool var_smtp_dummy_mail_auth;
bool var_smtp_cname_overr;
bool var_smtp_rand_addr;
bool var_frozen_delivered;
bool var_biff;
