bool var_tlsp_clnt_enforce_tls;
bool var_tlsp_tls_ask_ccert;
bool var_relay_before_rcpt_checks;
bool var_psc_enforce_tls;
bool var_threaded_bounce;
bool var_tlsp_tls_req_ccert;
bool var_tlsp_tls_enable_rpk;
bool var_append_dot_mydomain;
bool var_smtpd_hide_client_session;
bool var_smtp_tls_enf_sts_mx_pat;
bool var_smtp_tlsrpt_skip_reused_hs;
bool var_psc_use_tls;
bool var_reqtls_redact_dsn;
bool var_tlsp_clnt_use_tls;
bool var_enb_relocated_pfx;
bool var_tlsp_use_tls;
bool var_smtp_tlsrpt_enable;
bool var_psc_helo_required;
bool var_psc_disable_vrfy;
bool var_smtpd_req_deadline;
bool var_reqtls_esmtp_hdr;
bool var_tlsp_tls_set_sessid;
bool var_tlsp_enforce_tls;
bool var_log_tls_feature_status;
bool var_smtp_req_deadline;
